@@ -3,26 +3,11 @@
     Copyright (c) 2022 Itzsten
 */
 
-#include "Python.h"
-#include <Windows.h>
-#include <stdio.h>
+#include "utils.h"
 
 #pragma comment(lib, "User32.lib")
 #pragma comment(lib, "Advapi32.lib")
-
-#if PY_MAJOR_VERSION >= 3
-#define PyString_ToCharArr PyUnicode_AsUTF8
-#else
-#define PyString_ToCharArr PyString_AsString
-#endif
-
-BOOL WINAPI RaiseExceptionCheck(BOOL bSuccess) {
-    if (GetLastError() && (!bSuccess)) {
-        PyErr_SetFromWindowsErr(GetLastError());
-        return TRUE;
-    }
-    return FALSE;
-}
+#pragma comment(lib, "Shell32.lib")
 
 typedef struct tagPYENUMCHILDWINDOWPARAMS {
     PyObject* func;
@@ -732,7 +717,185 @@ static PyObject* PyFillRect(PyObject* self, PyObject* args) {
     return Py_BuildValue("O", PyBool_FromLong(res));
 }
 
+static PyObject* PyGetKeyboardLayout(PyObject* _, PyObject* args) {
+    DWORD thread;
+    if (!PyArg_ParseTuple(args, "k", &thread)) return NULL;
+    HKL res = GetKeyboardLayout(thread);
+    return Py_BuildValue("(lL)", LOWORD(res), HIWORD(res));
+}
+
+static PyObject* PyActivateKeyboardLayout(PyObject* _, PyObject* args) {
+    INT a;
+    UINT flags;
+    if (!PyArg_ParseTuple(args, "iI", &a, &flags)) return NULL;
+    HKL res = ActivateKeyboardLayout(a, flags);
+    if (RaiseExceptionCheck(res)) return NULL;
+    return Py_BuildValue("L", res);
+}
+
+static PyObject* PyBlockInput(PyObject* _, PyObject* args) {
+    PyObject* b;
+    if (!PyArg_ParseTuple(args, "O", &b)) return NULL;
+    if (!PyBool_Check(b)) {
+        PyErr_SetString(PyExc_TypeError, "argument 1: excepted bool object");
+        return NULL;
+    }
+    BOOL bBlock = PyObject_IsTrue(b);
+    BOOL res = BlockInput(bBlock);
+    if (RaiseExceptionCheck(res)) return NULL;
+    return Py_BuildValue("i", 0);
+}
+
+static PyObject* PyAbortSystemShutdownA(PyObject* _, PyObject* args) {
+    PyObject* computer;
+    CHAR* comp = NULL;
+    if (!PyArg_ParseTuple(args, "O", &computer)) return NULL;
+    if (computer != Py_None) comp = PyUnicode_AsUTF8(computer);
+    if (RaiseExceptionCheck(AbortSystemShutdownA(comp))) return NULL;
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyAbortSystemShutdownW(PyObject* _, PyObject* args) {
+    PyObject* computer;
+    WCHAR* comp = NULL;
+    if (!PyArg_ParseTuple(args, "O", &computer)) return NULL;
+    if (computer != Py_None) {
+        Py_ssize_t sz = PyUnicode_GetLength(computer);
+        comp = PyUnicode_AsWideCharString(computer, &sz);
+    }
+    if (RaiseExceptionCheck(AbortSystemShutdownW(comp))) return NULL;
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyGetConsoleWindow(PyObject* _, PyObject* args) {
+    HWND res = GetConsoleWindow();
+    return Py_BuildValue("O", res == NULL ? Py_None : PyLong_FromLongLong(res));
+}
+static PyObject* PyGetCommandLineA(PyObject* _, PyObject* args) {
+    return Py_BuildValue("s", GetCommandLineA());
+}
+static PyObject* PyGetCommandLineW(PyObject* _, PyObject* args) {
+    return Py_BuildValue("u", GetCommandLineW());
+}
+static PyObject* PyCommandLineToArgvW(PyObject* _, PyObject* args) {
+    WCHAR* cmdLine;
+    if (!PyArg_ParseTuple(args, "u", &cmdLine)) return NULL;
+
+    int argc;
+    LPCWSTR* argv = CommandLineToArgvW(cmdLine, &argc);
+    if (!argc) return Py_BuildValue("s", "");
+
+    PyObject* arr = PyTuple_New(argc);
+
+    for (INT i = 0; i < argc; i++) {
+        PyTuple_SET_ITEM(arr, i, Py_BuildValue("u", argv[i]));
+    }
+
+    return Py_BuildValue("O", arr);
+}
+static PyObject* PyCloseHandle(PyObject* _, PyObject* args) {
+    HANDLE h;
+    if (!PyArg_ParseTuple(args, "L", &h)) return NULL;
+    if (RaiseExceptionCheck(CloseHandle(h))) return NULL;
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyCopyFileA(PyObject* _, PyObject* args) {
+    CHAR* currentFile;
+    CHAR* newFile;
+    PyObject* boolFailIfExists;
+    if (!PyArg_ParseTuple(args, "ssO", &currentFile, &newFile, &boolFailIfExists)) return NULL;
+
+    if (!PyBool_Check(boolFailIfExists)) {
+        PyErr_SetString(PyExc_TypeError, "argument 3: excepted bool object");
+        return NULL;
+    }
+    BOOL bFailIfExist = PyObject_IsTrue(boolFailIfExists);
+    if (RaiseExceptionCheck(CopyFileA(currentFile, newFile, bFailIfExist))) return NULL;
+
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyCopyFileW(PyObject* _, PyObject* args) {
+    WCHAR* currentFile;
+    WCHAR* newFile;
+    PyObject* boolFailIfExists;
+    if (!PyArg_ParseTuple(args, "uuO", &currentFile, &newFile, &boolFailIfExists)) return NULL;
+
+    if (!PyBool_Check(boolFailIfExists)) {
+        PyErr_SetString(PyExc_TypeError, "argument 3: excepted bool object");
+        return NULL;
+    }
+    BOOL bFailIfExist = PyObject_IsTrue(boolFailIfExists);
+    if (RaiseExceptionCheck(CopyFileW(currentFile, newFile, bFailIfExist))) return NULL;
+
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyDeleteFileA(PyObject* _, PyObject* args) {
+    CHAR* targetFile;
+    if (!PyArg_ParseTuple(args, "s", &targetFile)) return NULL;
+    if (RaiseExceptionCheck(DeleteFileA(targetFile))) return NULL;
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyDeleteFileW(PyObject* _, PyObject* args) {
+    WCHAR* targetFile;
+    if (!PyArg_ParseTuple(args, "u", &targetFile)) return NULL;
+    if (RaiseExceptionCheck(DeleteFileW(targetFile))) return NULL;
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyGetComputerNameA(PyObject* _, PyObject* args) {
+    CHAR sz[MAX_COMPUTERNAME_LENGTH + 1];
+    DWORD len = sizeof(sz) / sizeof(CHAR);
+    if (RaiseExceptionCheck(GetComputerNameA(sz, &len))) return NULL;
+    return Py_BuildValue("s", sz);
+}
+static PyObject* PyGetComputerNameW(PyObject* _, PyObject* args) {
+    WCHAR buffer[MAX_COMPUTERNAME_LENGTH + 1];
+    DWORD sz = sizeof(buffer) / sizeof(WCHAR);
+    if (RaiseExceptionCheck(GetComputerNameW(buffer, &sz))) return NULL;
+    return Py_BuildValue("u", buffer);
+}
+static PyObject* PyGetComputerNameExA(PyObject* _, PyObject* args) {
+    COMPUTER_NAME_FORMAT targetName;
+    if (!PyArg_ParseTuple(args, "i", &targetName)) return NULL;
+    DWORD szRequired;
+    GetComputerNameExA(targetName, NULL, &szRequired);
+    LPSTR s = malloc(sizeof(CHAR) * szRequired);
+    if (RaiseExceptionCheck(GetComputerNameExA(targetName, s, &szRequired))) return NULL;
+    return Py_BuildValue("s", s);
+}
+static PyObject* PyGetComputerNameExW(PyObject* _, PyObject* args) {
+    COMPUTER_NAME_FORMAT targetName;
+    if (!PyArg_ParseTuple(args, "i", &targetName)) return NULL;
+    DWORD szRequired;
+    GetComputerNameExW(targetName, NULL, &szRequired);
+    LPWSTR s = malloc(sizeof(CHAR) * szRequired);
+    if (RaiseExceptionCheck(GetComputerNameExW(targetName, s, &szRequired))) return NULL;
+    return Py_BuildValue("u", s);
+}
+
 static PyMethodDef module_methods[] = {
+    { "GetComputerNameEx",  PyGetComputerNameExW, METH_VARARGS },
+    { "GetComputerNameExW", PyGetComputerNameExW, METH_VARARGS },
+    { "GetComputerNameExA", PyGetComputerNameExA, METH_VARARGS },
+    { "GetComputerName",  PyGetComputerNameW, METH_NOARGS },
+    { "GetComputerNameW", PyGetComputerNameW, METH_NOARGS },
+    { "GetComputerNameA", PyGetComputerNameA, METH_NOARGS },
+    { "DeleteFile",  PyDeleteFileW, METH_VARARGS },
+    { "DeleteFileW", PyDeleteFileW, METH_VARARGS },
+    { "DeleteFileA", PyDeleteFileA, METH_VARARGS },
+    { "CopyFileW", PyCopyFileW, METH_VARARGS },
+    { "CopyFileA", PyCopyFileA, METH_VARARGS },
+    { "CopyFile",  PyCopyFileW, METH_VARARGS },
+    { "CloseHandle", PyCloseHandle, METH_VARARGS },
+    { "CommandLineToArgvW", PyCommandLineToArgvW, METH_VARARGS },
+    { "CommandLineToArgv",  PyCommandLineToArgvW, METH_VARARGS },
+    { "GetCommandLine",  PyGetCommandLineW, METH_NOARGS },
+    { "GetCommandLineA", PyGetCommandLineA, METH_NOARGS },
+    { "GetCommandLineW", PyGetCommandLineW, METH_NOARGS },
+    { "GetConsoleWindow", PyGetConsoleWindow, METH_NOARGS },
+    { "AbortSystemShutdownA", PyAbortSystemShutdownA, METH_VARARGS },
+    { "AbortSystemShutdownW", PyAbortSystemShutdownW, METH_VARARGS },
+    { "AbortSystemShutdown",  PyAbortSystemShutdownW, METH_VARARGS },
+    { "BlockInput", PyBlockInput, METH_VARARGS },
+    { "ActivateKeyboardLayout", PyActivateKeyboardLayout, METH_VARARGS },
+    { "GetKeyboardLayout", PyGetKeyboardLayout, METH_VARARGS },
     { "GetDC", (PyCFunction)PyGetDC, METH_VARARGS },
     { "GetSystemMetrics", (PyCFunction)PyGetSystemMetrics, METH_VARARGS },
     { "GetDesktopWindow", (PyCFunction)PyGetDesktopWindow,  METH_NOARGS },
@@ -794,7 +957,7 @@ static PyMethodDef module_methods[] = {
 static struct PyModuleDef ModuleCombinations =
 {
     PyModuleDef_HEAD_INIT,
-    "WinAPY_user", /* name of module */
+    "winapy_user", /* name of module */
     NULL,
     -1,   /* size of per-interpreter state of the module, or -1 if the module keeps state in global variables. */
     module_methods
